@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 import pathlib
+from dataclasses import dataclass
+from enum import Enum
 from typing import TYPE_CHECKING, Iterable
 
 import numpy as np
@@ -447,9 +449,74 @@ class PrimSweep(Shape):
         return f"PrimSweep({self.name})"
 
 
-class PrimBSplineSurface(Shape):
-    def __init__(self, name, udeg, vdeg, ctrlpts: list[list]):
-        super().__init__(name, geom=None)
+class IfcBSplineSurfaceForm(Enum):
+    PLANE_SURF = "PLANE_SURF"
+    CYLINDRICAL_SURF = "CYLINDRICAL_SURF"
+    CONICAL_SURF = "CONICAL_SURF"
+    SPHERICAL_SURF = "SPHERICAL_SURF"
+    TOROIDAL_SURF = "TOROIDAL_SURF"
+    SURF_OF_REVOLUTION = "SURF_OF_REVOLUTION"
+    RULED_SURF = "RULED_SURF"
+    GENERALISED_CONE = "GENERALISED_CONE"
+    QUADRIC_SURF = "QUADRIC_SURF"
+    SURF_OF_LINEAR_EXTRUSION = "SURF_OF_LINEAR_EXTRUSION"
+    UNSPECIFIED = "UNSPECIFIED"
+
+
+@dataclass
+class BSplineSurfaceWithKnots:
+    """https://standards.buildingsmart.org/IFC/RELEASE/IFC4_3/lexical/IfcBSplineSurfaceWithKnots.htm"""
+
+    uDegree: int
+    vDegree: int
+    controlPointsList: list[list[float]]
+    surfaceForm: IfcBSplineSurfaceForm
+    uKnots: list[float]
+    vKnots: list[float]
+    uMultiplicities: list[int]
+    vMultiplicities: list[int]
+
+    def get_entities(self):
+        return {
+            "UDegree": self.uDegree,
+            "VDegree": self.vDegree,
+            "ControlPointsList": self.controlPointsList,
+            "SurfaceForm": self.surfaceForm.value,
+            "UClosed": False,
+            "VClosed": False,
+            "SelfIntersect": False,
+            "UKnots": self.uKnots,
+            "VKnots": self.vKnots,
+            "UMultiplicities": self.uMultiplicities,
+            "VMultiplicities": self.vMultiplicities,
+            "KnotSpec": "UNSPECIFIED",
+        }
+
+    def to_ifcopenshell(self, f):
+        from ada.ifc.utils import ifc_p
+
+        entities = self.get_entities()
+        entities["ControlPointsList"] = [[ifc_p(f, i) for i in x] for x in self.controlPointsList]
+        return f.create_entity("IfcBSplineSurfaceWithKnots", **entities)
+
+
+@dataclass
+class RationalBSplineSurfaceWithKnots(BSplineSurfaceWithKnots):
+    """https://standards.buildingsmart.org/IFC/RELEASE/IFC4_3/lexical/IfcRationalBSplineSurfaceWithKnots.htm"""
+
+    weightsData: list[list[float]]
+
+    def get_entities(self):
+        entities = super().get_entities()
+        entities["WeightsData"] = self.weightsData
+        return entities
+
+    def to_ifcopenshell(self, f):
+        from ada.ifc.utils import ifc_p
+
+        entities = self.get_entities()
+        entities["ControlPointsList"] = [[ifc_p(f, i[:3]) for i in x] for x in self.controlPointsList]
+        return f.create_entity("IfcRationalBSplineSurfaceWithKnots", **entities)
 
 
 class Penetration(BackendGeom):
