@@ -1,11 +1,10 @@
 from __future__ import annotations
 
-import logging
 import mmap
 import os
 import pathlib
 import re
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from itertools import chain
 from typing import TYPE_CHECKING, Dict, List, Union
 
@@ -14,6 +13,7 @@ import numpy as np
 from ada.concepts.containers import Nodes
 from ada.concepts.points import Node
 from ada.concepts.transforms import Rotation, Transform
+from ada.config import get_logger
 from ada.core.utils import Counter
 from ada.fem import (
     Bc,
@@ -41,7 +41,10 @@ part_name_counter = Counter(1, "Part")
 
 
 if TYPE_CHECKING:
-    from ada.concepts.levels import FEM, Assembly, Part
+    from ada.concepts.spatial import Assembly, Part
+    from ada.fem import FEM
+
+logger = get_logger()
 
 
 @dataclass
@@ -49,7 +52,7 @@ class InstanceData:
     part_ref: str
     instance_name: str
     instance_bulk: str
-    transform: Transform = Transform()
+    transform: Transform = field(default_factory=Transform)
 
 
 def read_fem(fem_file, fem_name=None) -> Assembly:
@@ -106,9 +109,12 @@ def read_fem(fem_file, fem_name=None) -> Assembly:
         assembly.fem.sets.link_data()
 
         update_connector_data(ass_sets, assembly.fem)
-
         assembly.fem.surfaces.update(get_surfaces_from_bulk(ass_sets, assembly.fem))
-        assembly.fem.constraints.update(get_constraints_from_inp(ass_sets, assembly.fem))
+
+        try:
+            assembly.fem.constraints.update(get_constraints_from_inp(ass_sets, assembly.fem))
+        except KeyError as e:
+            logger.error(e)
 
         assembly.fem.bcs += get_bcs_from_bulk(props_str, assembly.fem)
         assembly.fem.elements += get_mass_from_bulk(ass_sets, assembly.fem)
@@ -193,6 +199,8 @@ def get_fem_from_bulk_str(name, bulk_str, assembly: Assembly, instance_data: Ins
     from ada import FEM, Part
 
     instance_name = name if instance_data.instance_name is None else instance_data.instance_name
+    if name in assembly.parts.keys():
+        name = instance_name
     part = assembly.add_part(Part(name, fem=FEM(name=instance_name)))
     fem = part.fem
     fem.nodes = get_nodes_from_inp(bulk_str, fem)
@@ -235,7 +243,7 @@ def get_initial_conditions_from_lines(assembly: Assembly, bulk_str: str):
         try:
             dofs = int(ev[1])
         except BaseException as e:
-            logging.debug(e)
+            logger.debug(e)
             dofs = ev[1]
         if len(ev) > 2:
             magn = ev[2]
@@ -465,7 +473,7 @@ def get_bcs_from_bulk(bulk_str, fem: FEM) -> List[Bc]:
                 try:
                     dofs.append(int(ev[1]))
                 except BaseException as e:
-                    logging.debug(e)
+                    logger.debug(e)
                     dofs.append(ev[1])
                 if len(ev) > 3:
                     magn.append(ev[2])
@@ -520,7 +528,7 @@ def get_surfaces_from_bulk(bulk_str, parent):
         try:
             ref = str_to_int(msplit[0])
         except BaseException as e:
-            logging.debug(e)
+            logger.debug(e)
             ref = msplit[0].strip()
 
         return tuple([ref, msplit[1].strip()])
@@ -567,7 +575,6 @@ def get_surfaces_from_bulk(bulk_str, parent):
                     weight_factor = float(set_id_ref)
                 el_face_index = None
             else:
-
                 weight_factor = None
                 fem_set = parent.sets.get_elset_from_name(set_ref)
                 el_type = find_element_type_from_list(fem_set.members)
@@ -576,7 +583,7 @@ def get_surfaces_from_bulk(bulk_str, parent):
                 elif el_type == ElemType.SHELL:
                     el_face_index = -1 if set_id_ref == "SNEG" else 1
                 else:
-                    raise NotImplementedError("Importing surfaces for line elements is not yet supported")
+                    el_face_index = set_id_ref
         else:
             fem_set = None
             weight_factor = None
@@ -682,13 +689,13 @@ def get_constraints_from_inp(bulk_str: str, fem: FEM) -> Dict[str, Constraint]:
             try:
                 n1_ = str_to_int(m)
             except BaseException as e:
-                logging.debug(e)
+                logger.debug(e)
                 n1_ = get_set_from_assembly(m, fem, FemSet.TYPES.NSET)
 
             try:
                 n2_ = str_to_int(s)
             except BaseException as e:
-                logging.debug(e)
+                logger.debug(e)
                 n2_ = get_set_from_assembly(s, fem, FemSet.TYPES.NSET)
 
             mpc_dict[mpc_type].append((n1_, n2_))
